package Grammar;

use 5.008004;
use strict;
use warnings;
use Parse::Yapp;

require Exporter;

our @ISA = qw(Exporter);

our @EXPORT = qw( );

our $VERSION = '0.03';


sub Parse {
  my $grammar = shift;

  my $x = new Parse::Yapp::Parse;
  my $r = $x->Parse($grammar);

  # Supress not used keys
  delete @$r{'EXPECT', 'HEAD', 'PREC'};


  my $rules = $r->{RULES};
  shift @$rules; # supress super-rule

  # recompute production indices for each syntax variable
  my @vars = keys(%{$r->{NTERM}});
  for my $var (@vars) {
    my $lines = $r->{NTERM}{$var};
    @$lines = map { --$_ } @$lines;
  }

  #supress priority and code
  for my $rule (@$rules) {
   $#$rule = 1;
  }
  #supress token priorities
  my @tokens = keys(%{$r->{TERM}});
  $r->{TERM} = \@tokens;
  return $r;
}

1;
__END__

=head1 NAME

Grammar - Reads a yacc or yapp grammar and returns a reference to a hash 
          describing the grammar

=head1 SYNOPSIS

  use Grammar;
  my $x = Grammar::Parse($grammar);

=head1 DESCRIPTION

This module is written only for educational purposes for
the subject "Procesadores de Lenguajes" at ETSII, Universidad 
de La Laguna. This doc is for version 0.3. 

Function Grammar::Parse receives as input a string
containing the grammar and returns a reference to 
a hash describing the grammar.
The input grammar follows the rules of a yacc grammar.
The main difference with version 0.2 is that tail code
is included in the structure.

Follows an example. When the following script is executed

  $ cat -n grammar.pl
       1  #!/usr/bin/perl -w -I../lib
       2  use strict;
       3  use Grammar;
       4  use Data::Dumper;
       5
       6  sub usage {
       7    print <<"EOI";
       8  usage:
       9  $0 input_grammar
      10  EOI
      11    die "\n";
      12  }
      13
      14  usage() unless @ARGV;
      15  my $filename = shift;
      16
      17  local $/ = undef;
      18  open FILE, "$filename";
      19  my $grammar = <FILE>;
      20  my $x = Grammar::Parse($grammar);

using as input the following grammar:

  Grammar/03/scripts$ cat -n aSb.yp
       1  %%
       2  S:
       3      |   'a' S 'b'
       4  ;
       5  %%
       6
       7  sub Lex {
       8    local $_ = shift; # input
       9    my @tokens;
      10
      11
      12    while ($_) {
      13      s/^\s*//; # fuera blancos
      14      push @tokens, $1, $1 if s/^(.)//s
      15    }
      16    @tokens;
      17  }
      18
      19  sub main {
      20    my $filename = shift;
      21    my $input;
      22
      23    if (defined($filename)) {
      24      local $/ = undef;
      25      open FILE, $filename or die "No se pudo abrir $filename\n";
      26      $input = <FILE>;
      27      close(FILE);
      28    }
      29    else { $input = <STDIN> }
      30
      31    my @tokens = Lex($input);
      32    Parse(@tokens); # Llamada al analizador generado
      33    print "Sintcticamente correcto\n";
      34  }


the following data structure is generated:

Grammar/03/scripts$ grammar.pl aSb.yp
  $VAR1 = {
            'SYMS' => { 'S' => 2, '\'b\'' => 3, '\'a\'' => 3 },
            'NULL' => { 'S' => 1 },
            'RULES' => [
                         [ 'S', [] ],
                         [ 'S', [ '\'a\'', 'S', '\'b\'' ] ]
                       ],
            'START' => 'S',
            'TERM' => [ '\'b\'', '\'a\'' ],
            'NTERM' => { 'S' => [ 0, 1 ] }
            'TAIL' => [ '

  sub Lex {
    local $_ = shift; # input
    my @tokens;


    while ($_) {
      s/^\\s*//; # fuera blancos
      push @tokens, $1, $1 if s/^(.)//s
    }
    @tokens;
  }

  sub main {
    my $filename = shift;
    my $input;

    if (defined($filename)) {
      local $/ = undef;
      open FILE, $filename or die "No se pudo abrir $filename\\n";
      $input = <FILE>;
      close(FILE);
    }
    else { $input = <STDIN> }

    my @tokens = Lex($input);
    my $ok = Parse(@tokens); # Llamada al analizador generado
    print "Sintcticamente correcto\\n" if $ok;
  }

  ', 5 ], # lnea en la que est el segundo %%
  };




=head2 EXPORT

Nothing

=head1 DEPENDENCIES

The module depends on Parse::Yapp

=head1 SEE ALSO

http://nereida.deioc.ull.es/~lhp/perlexamples/ 

=head1 AUTHOR

Casiano Rodriguez Leon

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by Casiano Rodriguez Leon

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.


=cut
